@@ -1,3 +1,4 @@
+import astropy
 from astropy.io import fits
 from astropy.nddata import NDData
 from astropy.stats import sigma_clipped_stats
@@ -12,7 +13,13 @@ from spike import tools
 import subprocess
 import urllib
 import warnings
-import webbpsf
+
+try: #for version control reasons
+	import stpsf as webbpsf
+	is_stpsf = True
+except:
+	import webbpsf
+	is_stpsf = False # for version control
 
 def warning_on_one_line(message, category, filename, lineno, file=None, line=None):
 	return '%s:%s: %s: %s\n' % (filename, lineno, category.__name__, message)
@@ -52,7 +59,7 @@ plate_scale = {'ACS/WFC':0.05, 'ACS/HRC':0.025 , 'WFC3/IR':0.13, 'WFC3/UVIS':0.0
 
 
 def tinypsf(coords, img, imcam, pos, plot = False, verbose = False, writeto = True,
-	ebmv = None, av = None, wmag = None,
+	clobber = False, ebmv = None, av = None, wmag = None,
 	jitter = None, major = None, minor = None, angle = None,
 	specchoice = 'blackbody', listchoice = 'G5V', temp = 6000., 
 	specalpha = 1., specbeta = 1., fov_arcsec = 6., despace = 0.):
@@ -63,16 +70,20 @@ def tinypsf(coords, img, imcam, pos, plot = False, verbose = False, writeto = Tr
 	Tiny Tim PSF model, please use spike.psfgen.tinygillispsf() instead.
 
 	Parameters:
-		coords (astropy skycoord object): Coordinates of object of interest or list of skycoord objects.
+		coords (str or astropy skycoord object): Coordinates of object of interest.
 		img (str): Path to image for which PSF is generated.
 		imcam (str): Specification of instrument/camera used to capture the images (e.g., 'ACS/WFC', 'WFC3/IR', 'WFPC', 'WFPC2').
-			For 'WFPC' and 'WFPC2', the camera is selecte by-chip and should not be specified here.
-		pos (list): Location of object of interest (spatial and spectral).[X, Y, chip, filter]
-		plot (bool): If True, saves .pngs of the model PSFs.
+			For 'WFPC' and 'WFPC2', the camera is selected by-chip and should not be specified here.
+		pos (list): Location of object of interest (spatial and spectral). [X, Y, chip, filter]
+			If None, will find location based on coordinates and instrument/camera.
+		plot (bool): If True, saves .pngs of the model PSFs. (Not affected by clobber; 
+			images with the same name are overwritten by default.)
 		verbose (bool): If True, prints progress messages.
 		writeto (bool): If True, will write 2D model PSF (differentiated with '_topsf' 
 			suffix) and will amend relevant image WCS information/remove extraneous extensions.
 			This is in addition to the 2D PSF model saved by TinyTim.
+		clobber (bool): If True, will overwrite existing files with the same name on save.
+			(Default state -- clobber = False -- is recommended.)
 		ebmv (float): Interstellar extinction, specified by reddening in mag. 
 			If specified, av should not be.
 		av (float): Interstellar extinction, specified by visual extinction in mag. 
@@ -91,13 +102,26 @@ def tinypsf(coords, img, imcam, pos, plot = False, verbose = False, writeto = Tr
 		temp (float): Temperature of blackbody spectrum in K.
 		specalpha (float): Spectral index alpha for F(nu)~nu^alpha.
 		specbeta (float): Spectral index alpha for F(lambda)~lambda^beta.
-		fov_arcsec (float): Diameter of model PSF image in arcsec.
+		fov_arcsec (float): "Diameter" of model PSF image in arcsec.
 		despace (float): Focus, secondary mirror despace in micron. Scaled by 0.011 and added to
 			the 4th Zernike polynomial.
 
 	Returns:
 		TinyTim model PSF
 	"""
+
+	if type(coords) == str:
+		coords = tools.objloc(coords)
+
+	if not pos:
+		im_cam = imcam.split('/')
+		inst = im_cam[0]
+		if len(im_cam) == 1:
+			camera = None
+		if len(im_cam) == 2:
+			camera = im_cam[1]
+		pos = tools.checkpixloc(coords, img, inst, camera)
+
 	if not TINY_PATH:
 		# this is a warning and not an error on the off chance that the TinyTim executables are 
 		# in the working directory
@@ -145,12 +169,12 @@ def tinypsf(coords, img, imcam, pos, plot = False, verbose = False, writeto = Tr
 		specparam = specbeta
 
 	coordstring = str(coords.ra)
-	if coords.dec.deg > 0:
-		coordstring += '+'+str(coords.dec)
 	if coords.dec.deg >= 0:
+		coordstring += '+'+str(coords.dec)
+	if coords.dec.deg < 0:
 		coordstring += str(coords.dec)
 
-	modname = img.replace('.fits', coordstring+'_%s'%pos[3]+'_psf')
+	modname = img.replace('.fits', '_'+coordstring+'_%s'%pos[3]+'_psf')
 
 	if imcam in ['ACS/WFC', 'WFC3/UVIS']:
 		command_list = [tinyparams['imcam'][imcam], pos[2], '%i %i'%(pos[0], pos[1]), 
@@ -222,15 +246,15 @@ def tinypsf(coords, img, imcam, pos, plot = False, verbose = False, writeto = Tr
 	if writeto:
 		if verbose:
 			print('Writing to %s.fits.'%modname.replace('_psf', '_topsf'))
-		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'TinyTim')
+		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'TinyTim', clobber = clobber)
 
 
 	return psfmodel
 
 
 
-def tinygillispsf(coords, img, imcam, pos, plot = False, verbose = False, keep = False, 
-	writeto = True, specchoice = 'blackbody', listchoice = 'G5V', temp = 6000., specalpha = 1., 
+def tinygillispsf(coords, img, imcam, pos, plot = False, verbose = False, keep = False, writeto = True, 
+	clobber = False, specchoice = 'blackbody', listchoice = 'G5V', temp = 6000., specalpha = 1., 
 	specbeta = 1., fov_arcsec = 6., despace = 0., sample = 1., linearfit = False, regrid = True):
 	"""
 	Generate HST PSFs using TinyTim and the parameter changes laid out in Gillis et al. (2020), 
@@ -242,16 +266,20 @@ def tinygillispsf(coords, img, imcam, pos, plot = False, verbose = False, keep =
 	it will be removed after use.
 
 	Parameters:
-		coords (astropy skycoord object): Coordinates of object of interest or list of skycoord objects.
+		coords (str or astropy skycoord object): Coordinates of object of interest.
 		img (str): Path to image for which PSF is generated.
 		imcam (str): 'ACS/WFC' is the only recommended instrument/camera combination for this PSF generation method.
 		pos (list): Location of object of interest (spatial and spectral).[X, Y, chip, filter]
-		plot (bool): If True, saves .pngs of the model PSFs.
+			If None, will find location based on coordinates and instrument/camera.
+		plot (bool): If True, saves .pngs of the model PSFs. (Not affected by clobber; 
+			images with the same name are overwritten by default.)
 		verbose (bool): If True, prints progress messages.
 		keep (bool): If True, retains make_psf.py (Gillis et al. 2020)
 		writeto (bool): If True, will write 2D model PSF (differentiated with '_topsf' 
 			suffix) and will amend relevant image WCS information/remove extraneous extensions.
 			This is in addition to the 2D PSF model saved by TinyTim.
+		clobber (bool): If True, will overwrite existing files with the same name on save.
+			(Default state -- clobber = False -- is recommended.)
 		specchoice (str): 'list', 'blackbody', 'plaw_nu', 'plaw_lam' -- if 'list', must also specify
 			listchoice; if 'blackbody', must also specify temp; if 'plaw_nu', must also specify specalpha;
 			and if 'plaw_lam', must also specify specbeta.
@@ -260,7 +288,7 @@ def tinygillispsf(coords, img, imcam, pos, plot = False, verbose = False, keep =
 		temp (float): Temperature of blackbody spectrum in K.
 		specalpha (float): Spectral index alpha for F(nu)~nu^alpha.
 		specbeta (float): Spectral index alpha for F(lambda)~lambda^beta.
-		fov_arcsec (float): Diameter of model PSF image in arcsec.
+		fov_arcsec (float): "Diameter" of model PSF image in arcsec.
 		despace (float): Focus, secondary mirror despace in micron. Scaled by 0.011 and added to
 			the 4th Zernike polynomial.
 		sample (float): Factor by which to undersample the PSF. Default is not to undersample.
@@ -271,6 +299,18 @@ def tinygillispsf(coords, img, imcam, pos, plot = False, verbose = False, keep =
 		TinyTim model PSF using amended Gillis et al. (2020) parameters
 	"""
 
+	if type(coords) == str:
+		coords = tools.objloc(coords)
+
+	if not pos:
+		im_cam = imcam.split('/')
+		inst = im_cam[0]
+		if len(im_cam) == 1:
+			camera = None
+		if len(im_cam) == 2:
+			camera = im_cam[1]
+		pos = tools.checkpixloc(coords, img, inst, camera)
+
 	if not TINY_PATH:
 		# this is a warning and not an error on the off chance that the TinyTim executables are 
 		# in the working directory
@@ -305,9 +345,9 @@ def tinygillispsf(coords, img, imcam, pos, plot = False, verbose = False, keep =
 	from make_psf import make_subsampled_model_psf
 
 	coordstring = str(coords.ra)
-	if coords.dec.deg > 0:
-		coordstring += '+'+str(coords.dec)
 	if coords.dec.deg >= 0:
+		coordstring += '+'+str(coords.dec)
+	if coords.dec.deg < 0:
 		coordstring += str(coords.dec)
 		
 	modname = img.replace('.fits', coordstring+'_%s'%pos[3]+'_psf')
@@ -363,13 +403,13 @@ def tinygillispsf(coords, img, imcam, pos, plot = False, verbose = False, keep =
 	if writeto:
 		if verbose:
 			print('Writing to %s.fits.'%modname.replace('_psf', '_topsf'))
-		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'TinyTim (Gillis+ mod)')
+		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'TinyTim (Gillis+ mod)', clobber = clobber)
 
 	return psfmodel
 
 
 def stdpsf(coords, img, imcam, pos, plot = False, verbose = False, 
-	writeto = True, fov_arcsec = 6, norm = 1, regrid = True):
+	writeto = True, clobber = False, fov_arcsec = 6, norm = 1, regrid = True):
 	"""
 	Coordinate-specific PSFs from STDPSF model grids for HST, JWST.
 
@@ -377,17 +417,21 @@ def stdpsf(coords, img, imcam, pos, plot = False, verbose = False,
 	https://www.stsci.edu/~jayander/JWST1PASS/LIB/PSFs/STDPSFs/.
 
 	Parameters:
-		coords (astropy skycoord object): Coordinates of object of interest or list of skycoord objects.
+		coords (str or astropy skycoord object): Coordinates of object of interest.
 		img (str): Path to image for which PSF is generated.
 		imcam (str): Specification of instrument/camera used to capture the images (e.g., 'ACS/WFC', 'WFC3/IR', 'WFPC', 
 			'WFPC2', 'MIRI', 'NIRCAM', 'NIRISS/Imaging'). For 'WFPC' and 'WFPC2', the camera is selected by-chip and 
 			should not be specified here. If 'NIRISS' specified alone, assumes the imaging mode.
 		pos (list): Location of object of interest (spatial and spectral).[X, Y, chip, filter]
-		plot (bool): If True, saves .pngs of the model PSFs.
+			If None, will find location based on coordinates and instrument/camera.
+		plot (bool): If True, saves .pngs of the model PSFs. (Not affected by clobber; 
+			images with the same name are overwritten by default.)
 		verbose (bool): If True, prints progress messages.
 		writeto (bool): If True, will write 2D model PSF (differentiated with '_topsf' 
 			suffix) and will amend relevant image WCS information/remove extraneous extensions.
-		fov_arcsec (float): Diameter of model PSF image in arcsec.
+		clobber (bool): If True, will overwrite existing files with the same name on save.
+			(Default state -- clobber = False -- is recommended.)
+		fov_arcsec (float): "Diameter" of model PSF image in arcsec.
 		norm (float): Flux normalization for output PSF model.
 
 	Returns:
@@ -395,6 +439,18 @@ def stdpsf(coords, img, imcam, pos, plot = False, verbose = False,
 
 	"""
 
+	if type(coords) == str:
+		coords = tools.objloc(coords)
+
+	if not pos:
+		im_cam = imcam.split('/')
+		inst = im_cam[0]
+		if len(im_cam) == 1:
+			camera = None
+		if len(im_cam) == 2:
+			camera = im_cam[1]
+		pos = tools.checkpixloc(coords, img, inst, camera)
+
 	# build the url that points to the STDPSF
 	imcamurl = imcam.replace('/', '')
 	if imcamurl == 'WFC3UVIS':
@@ -419,6 +475,8 @@ def stdpsf(coords, img, imcam, pos, plot = False, verbose = False,
 			url += '_SM3.fits'
 		elif (imcam == 'WFC3/IR') and (pos[3] == 'F139M'):
 			url += '_1x1.fits'
+		elif (imcam == 'WFC3/UVIS') and (pos[3] == 'F475W'):
+			url += 'x.fits'
 		else:
 			url += '.fits'
 
@@ -436,24 +494,18 @@ def stdpsf(coords, img, imcam, pos, plot = False, verbose = False,
 
 			if pos[3] in ['F070W', 'F090W', 'F115W', 'F140M', 'F150W', 'F182M', 'F200W'
 							'F210M', 'F212N']:
-				url = baseurl+imcamurl+'/SWC/%s/STDPSF_%s_%s.fits'%(pos[3], pos[3], pos[2])
-
-				# fix since photutils GriddedPSFModel reads the detector from the filename
-				# and expects a particular structure -- once fixed in photutils, will remove
-				# this hacky solution
-				newname = 'STDPSF_%s_%s.fits'%(pos[2], pos[3])
-				os.system('curl "%s" --output "%s"'%(url, newname))
+				url = baseurl+imcamurl+'/SWC/%s/STDPSF_%s_%s.fits'%(pos[3], pos[2], pos[3])
 
 	det = None #set detector for photutils
-	if imcam in ['WFPC2', 'ACS/WFC']:
+	if imcam in ['WFPC2', 'ACS/WFC', 'WFC3/UVIS']:
 		det = pos[2]
 	if (imcam == 'NIRCAM') and (pos[2] not in ['NGCA5', 'NRCB5']):
 		det = stdpsf_jwdet[pos[2]]
 
 	coordstring = str(coords.ra)
-	if coords.dec.deg > 0:
-		coordstring += '+'+str(coords.dec)
 	if coords.dec.deg >= 0:
+		coordstring += '+'+str(coords.dec)
+	if coords.dec.deg < 0:
 		coordstring += str(coords.dec)
 
 	modname = img.replace('.fits', '_'+coordstring+'_%s'%pos[3]+'_psf')
@@ -486,15 +538,8 @@ def stdpsf(coords, img, imcam, pos, plot = False, verbose = False,
 
 	#preferred equivalent to using photutils.psf.stdpsf_reader directly
 
-	if (imcam == 'NIRCAM') and (pos[3] in ['F070W', 'F090W', 'F115W', 'F140M', 'F150W', 
-		'F182M', 'F200W', 'F210M', 'F212N']):
-		# part of hacky solution for photutils NIRCam bug
-		model = GriddedPSFModel.read(filename = newname, detector_id = det, format= 'stdpsf')
-
-	else:
-		model = GriddedPSFModel.read(filename = url, detector_id = det, format= 'stdpsf')
+	model = GriddedPSFModel.read(filename = url, detector_id = det, format= 'stdpsf')
 	
-
 	if verbose:
 		print('Finished reading STDPSF grid, generating PSF model')
 	psfmodel = model.evaluate(x = x, y = y, flux = norm, x_0 = int(pos[0]), y_0 = int(pos[1]))
@@ -512,103 +557,92 @@ def stdpsf(coords, img, imcam, pos, plot = False, verbose = False,
 	if writeto:
 		if verbose:
 			print('Writing to %s.fits.'%modname.replace('_psf', '_topsf'))
-		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'STDPSFs')
+		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'STDPSFs', clobber = clobber)
 
 
-	if (imcam == 'NIRCAM') and (pos[3] in ['F070W', 'F090W', 'F115W', 'F140M', 'F150W', 
-		'F182M', 'F200W', 'F210M', 'F212N']):
-		# clean up from hacky solution
-		os.system('rm %s'%newname)
-
 	return psfmodel
 
 
 def jwpsf(coords, img, imcam, pos, plot = False, verbose = False, writeto = True,
-	fov = 6, sample = 1., regrid = True, image_mask = None, pupil_mask = None,
-	**calckwargs):
+	clobber = False, fov_arcsec = 6, sample = 4, regrid = True, image_mask = None, 
+	pupil_mask = None, savefull = False, **calckwargs):
 	"""
-	Generate JWST and Roman PSFs using WebbPSF.
+	Generate JWST and Roman PSFs using WebbPSF/STPSF. Note: reference to the WebbPSF name is 
+	maintained here in lieu of STPSF to avoid confusion with the generation of empirical STDPSFs.
 
 	Parameters:
-		coords (astropy skycoord object): Coordinates of object of interest or list of skycoord objects.
+		coords (str or astropy skycoord object): Coordinates of object of interest.
 		img (str): Path to image for which PSF is generated.
 		imcam (str): Specification of instrument/camera used to capture the images (e.g., 'ACS/WFC', 'WFC3/IR', 'WFPC', 
 			'WFPC2', 'MIRI', 'NIRCAM', 'NIRISS/Imaging'). For 'WFPC' and 'WFPC2', the camera is selected by-chip and 
 			should not be specified here. If 'NIRISS' specified alone, assumes the imaging mode.
 		pos (list): Location of object of interest (spatial and spectral).[X, Y, chip, filter]
-		plot (bool): If True, saves .pngs of the model PSFs.
+			If None, will find location based on coordinates and instrument/camera.
+		plot (bool): If True, saves .pngs of the model PSFs. (Not affected by clobber; 
+			images with the same name are overwritten by default.)
 		verbose (bool): If True, prints progress messages.
 		writeto (bool): If True, will write 2D model PSF (differentiated with '_topsf' 
 			suffix) and will amend relevant image WCS information/remove extraneous extensions.
 			This is in addition to the 2D PSF models saved by WebbPSF (which will be saved as img_psf.fits).
-		fov_arcsec (float): Diameter of model PSF image in arcsec.
+		clobber (bool): If True, will overwrite existing files with the same name on save.
+			(Default state -- clobber = False -- is recommended.)
+		fov_arcsec (float): "Diameter" of model PSF image in arcsec.
 		sample (float): Factor by which to oversample the PSF.
 		regrid (bool): If True, will (interpolate and) regrid model PSF to image pixel scale.
 		image_mask (str): Image mask argument for WebbPSF.
 		pupil_mask (str): Pupil mask argument for WebbPSF.
+		savefull (bool): If True, save the full multi-extension WebbPSF/STPSF output.
 		**calckwargs: Additional arguments for calc_psf() -- see 
-			https://webbpsf.readthedocs.io/en/latest/api/webbpsf.JWInstrument.html#webbpsf.JWInstrument.calc_psf
-			Should be fed to the spike.psf.jwst/roman in kwargs as a dictionary called calckwargs.
+			https://stpsf.readthedocs.io/en/latest/usage.html#.
+			Should be fed to spike.psf.jwst/roman in kwargs as a dictionary called calckwargs.
 
 	Returns:
 		WebbPSF model PSF
 
 	"""
+
+	if type(coords) == str:
+		coords = tools.objloc(coords)
+
+	if not pos:
+		im_cam = imcam.split('/')
+		inst = im_cam[0]
+		if len(im_cam) == 1:
+			camera = None
+		if len(im_cam) == 2:
+			camera = im_cam[1]
+		pos = tools.checkpixloc(coords, img, inst, camera)
+
+
 	x, y, chip, filt = pos
 
 	coordstring = str(coords.ra)
-	if coords.dec.deg > 0:
-		coordstring += '+'+str(coords.dec)
 	if coords.dec.deg >= 0:
+		coordstring += '+'+str(coords.dec)
+	if coords.dec.deg < 0:
 		coordstring += str(coords.dec)
 
 	modname = img.replace('.fits', '_'+coordstring+'_%s'%pos[3]+'_psf')
 
-	if imcam.upper() == 'NIRCAM':
-		psf = webbpsf.NIRCam()
-
-	if imcam.upper() == 'MIRI':
-		psf = webbpsf.MIRI()
-
-	if imcam.upper() == 'NIRISS':
-		psf = webbpsf.NIRISS()
-
-	if imcam.upper() == 'WFI':
-		psf = webbpsf.roman.WFI()
-
-	if imcam.upper() == 'CGI':
-		psf = webbpsf.roman.RomanCoronagraph()
-		warnings.warn("WebbPSF Roman CGI development halted in 2017. Use with caution.", Warning, stacklevel = 2) 
-
-	if imcam.upper() in ['NIRCAM', 'WFI']:
-		psf.detector = chip
-
-
-	psf.filter = filt
-	psf.detector_position = (x, y)
-
-	if image_mask:
-		psf.image_mask = image_mask
-
-	if pupil_mask:
-		psf.pupil_mask = pupil_mask
-
-	#read aperture name from FITS header
-	imfits = fits.open(img)
-	aperturename = imfits[0].header['APERNAME']
-	psf.aperturename = aperturename
+	psf = webbpsf.setup_sim_to_match_file(img)
+	psf.detector_position = (x, y) #set detector_position
 
 	if verbose:
 		print('Producing PSF model')
-	psfmod = psf.calc_psf(fov_arcsec = fov, oversample = sample, **calckwargs)
+	psfmod = psf.calc_psf(fov_arcsec = fov_arcsec, oversample = sample, **calckwargs)
 
-	psfmodel = psfmod[3].data
+	psfmodel = psfmod['DET_DIST'].data
 
-	if regrid:
-		if sample != 1.:
-			if verbose:
-				print('Regridding PSF to instrument resolution')
-			psfmodel = tools.regridarr(psfmodel, sample)
+	if savefull:
+		if is_stpsf:
+			code_name = 'STPSF'
+		if not is_stpsf:
+			code_name = 'WebbPSF'
+
+		if verbose:
+			print('Saving full %s output'%code_name)
+		
+		psfmod.writeto(modname.replace('_psf', '_%s'%code_name)+'.fits', overwrite = clobber)
 
 
 	if plot:
@@ -625,14 +659,15 @@ def jwpsf(coords, img, imcam, pos, plot = False, verbose = False, writeto = True
 	if writeto:
 		if verbose:
 			print('Writing to %s.fits.'%modname.replace('_psf', '_topsf'))
-		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'WebbPSF')
+		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'WebbPSF', clobber = clobber)
 
 
 	return psfmodel
 
 def effpsf(coords, img, imcam, pos, plot = False, verbose = False, mask = True, writeto = True, 
-	fov_arcsec = 6, norm = 1., starselect = 'DAO', starselectargs = {'fwhm':10}, thresh = 125,
-	usermask = None, epsfargs = {'oversampling':1, 'progress_bar':True, 'maxiters':10}):
+	clobber = False, fov_arcsec = 6, norm = 1., starselect = 'DAO', starselectargs = {'fwhm':10}, 
+	thresh = 125, usermask = None, maskval = None, 
+	epsfargs = {'oversampling':1, 'progress_bar':True, 'maxiters':10}):
 	"""
 	Generate PSFs using the empirical photutils.epsf routine. 
 
@@ -641,18 +676,22 @@ def effpsf(coords, img, imcam, pos, plot = False, verbose = False, mask = True,
 	PSF may be lower.
 
 	Parameters:
-		coords (astropy skycoord object): Coordinates of object of interest or list of skycoord objects.
+		coords (str or astropy skycoord object): Coordinates of object of interest.
 		img (str): Path to image for which PSF is generated.
 		imcam (str): Specification of instrument/camera used to capture the images (e.g., 'ACS/WFC', 'WFC3/IR', 'WFPC', 
-			'WFPC2', 'MIRI', 'NIRCAM', 'NIRISS/Imaging'). For 'WFPC' and 'WFPC2', the camera is selecte by-chip and 
+			'WFPC2', 'MIRI', 'NIRCAM', 'NIRISS/Imaging'). For 'WFPC' and 'WFPC2', the camera is selected by-chip and 
 			should not be specified here.
-		pos (list): Location of object of interest (spatial and spectral).[X, Y, chip, filter]
-		plot (bool): If True, saves .pngs of the model PSFs.
+		pos (list): Location of object of interest (spatial and spectral). [X, Y, chip, filter]
+			If None, will find location based on coordinates and instrument/camera.
+		plot (bool): If True, saves .pngs of the model PSFs. (Not affected by clobber; 
+			images with the same name are overwritten by default.)
 		verbose (bool): If True, prints progress messages.
 		mask (bool): If True, uses data quality array to mask some pixels.
 		writeto (bool): If True, will write 2D model PSF (differentiated with '_topsf' 
 			suffix) and will amend relevant image WCS information/remove extraneous extensions.
-		fov_arcsec (float): Diameter of model PSF image in arcsec.
+		clobber (bool): If True, will overwrite existing files with the same name on save.
+			(Default state -- clobber = False -- is recommended.)
+		fov_arcsec (float): "Diameter" of model PSF image in arcsec.
 		norm (float): Flux normalization for output PSF model.
 		starselect (str): 'DAO', 'IRAF', or 'peak', which use DAOStarFinder, IRAFStarFinder, and 
 			find_peaks from photutils respectively.
@@ -662,6 +701,8 @@ def effpsf(coords, img, imcam, pos, plot = False, verbose = False, mask = True,
 			which stars are used to build the model.
 		usermask (arr): If mask = True, used in addition to the DQ array to mask bad pixels. Must be
 			the same dimensions as the data array.
+		maskval (float): Value to assign to masked pixels. If None, default set by star seletion method. 
+			Otherwise, can be np.nan or number. Default for 'IRAF' is np.nan, default for 'DAO' is 0.
 		epsfargs (dict): Keyword arguments for the EPSFBuilder. Default in spike is to not oversample
 			the PSF, but the regridding is all handled during the creation of the coord-specific model.
 
@@ -670,6 +711,18 @@ def effpsf(coords, img, imcam, pos, plot = False, verbose = False, mask = True,
 	
 	"""
 
+	if type(coords) == str:
+		coords = tools.objloc(coords)
+
+	if not pos:
+		im_cam = imcam.split('/')
+		inst = im_cam[0]
+		if len(im_cam) == 1:
+			camera = None
+		if len(im_cam) == 2:
+			camera = im_cam[1]
+		pos = tools.checkpixloc(coords, img, inst, camera)
+
 	ext = 1 #read data from relevant SCI extension
 	extv = 1
 	if (imcam in ['ACS/WFC', 'WFC3/UVIS']) and (pos[2] == 1):
@@ -684,9 +737,9 @@ def effpsf(coords, img, imcam, pos, plot = False, verbose = False, mask = True,
 
 
 	coordstring = str(coords.ra)
-	if coords.dec.deg > 0:
-		coordstring += '+'+str(coords.dec)
 	if coords.dec.deg >= 0:
+		coordstring += '+'+str(coords.dec)
+	if coords.dec.deg < 0:
 		coordstring += str(coords.dec)
 
 	modname = img.replace('.fits', '_'+coordstring+'_%s'%pos[3]+'_psf')
@@ -714,24 +767,34 @@ def effpsf(coords, img, imcam, pos, plot = False, verbose = False, mask = True,
 
 	if starselect.upper() == 'DAO':
 		# take default FWHM to be 4x the detector plate scale, can overwrite with starselectargs
+
 		find = DAOStarFinder(threshold = thresh*std, **starselectargs)
+		if not maskval:
+			maskval = 0
 
 	if starselect.upper() == 'IRAF':
-		find = IRAFStarFinder(threshold = thresh*std,  **starselectargs)
+		## from tests, nan masks work best with IRAF
+		# suggested thresh = 3 here, as function of masking
 
+		mean, median, std = sigma_clipped_stats(dat, sigma=3.0)
+
+		find = IRAFStarFinder(threshold = thresh*std,  **starselectargs)
+		if not maskval:
+			maskval = np.nan	
+
+	if verbose:
+		print('Identifying stars to use in ePSF')
+	
 	if mask:
 		maskarr = fits.open(img)[('DQ', extv)].data
-		dat[maskarr > 0] = 0 # only retain good pixels
+		dat[maskarr > 0] = maskval # only retain good pixels
 		maskarr[maskarr > 0] = True
 		if usermask:
-			dat[usermask] = 0
-		if verbose:
-			print('Identifying stars to use in ePSF')
-		sources = find(dat, mask = maskarr)
+			dat[usermask] = maskval
+			maskarr[usermask] = True
+		sources = find(dat, maskarr)
 
 	if not mask:
-		if verbose:
-			print('Identifying stars to use in ePSF')
 		sources = find(dat)
 
 	exsize = int(2 * ((fov_arcsec/plate_scale[pixkey])//2) + 1) #size of extraction box
@@ -742,7 +805,7 @@ def effpsf(coords, img, imcam, pos, plot = False, verbose = False, mask = True,
 	tab = Table()
 	tab['x'] = xs[exmask]
 	tab['y'] = ys[exmask]
-	nddata = NDData(data = dat - median) 
+	nddata = NDData(data = dat - np.nanmedian(dat)) 
 	if verbose:
 		print('Beginning stellar extraction.')
 	stars = extract_stars(nddata, tab, size = exsize)
@@ -785,13 +848,13 @@ def effpsf(coords, img, imcam, pos, plot = False, verbose = False, mask = True,
 	if writeto:
 		if verbose:
 			print('Writing to %s.fits.'%modname.replace('_psf', '_topsf'))
-		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'ePSFs')
+		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'ePSFs', clobber = clobber)
 
 	return psfmodel
 
 
 def psfex(coords, img, imcam, pos, plot = False, verbose = False, writeto = True, 
-	savepsfex = False, crclean = True, seconf = None, psfconf = None, regrid = True, 
+	clobber = False, savepsfex = False, seconf = None, psfconf = None, regrid = True,
 	mask = True, maskparams = {}):
 	"""
 	Generate PSFs using PSFEx.
@@ -802,18 +865,22 @@ def psfex(coords, img, imcam, pos, plot = False, verbose = False, writeto = True
 	default naming scheme for .cat and .psf files and subsequent steps to rename them.)
 
 	Parameters:
-		coords (astropy skycoord object): Coordinates of object of interest or list of skycoord objects.
+		coords (str or astropy skycoord object): Coordinates of object of interest.
 		imcam (str): Specification of instrument/camera used to capture the images (e.g., 'ACS/WFC', 'WFC3/IR', 'WFPC', 
 			'WFPC2', 'MIRI', 'NIRCAM', 'NIRISS/Imaging'). For 'WFPC' and 'WFPC2', the camera is selected by-chip and 
 			should not be specified here.
 		img (str): Path to image for which PSF is generated.
 		pos (list): [X, Y, chip, filter] as output from spike.tools.checkpixloc.
-		plot (bool): If True, saves .pngs of the model PSFs.
+			If None, will find location based on coordinates and instrument/camera.
+		plot (bool): If True, saves .pngs of the model PSFs. (Not affected by clobber; 
+			images with the same name are overwritten by default.)
 		verbose (bool): If True, prints progress messages.
 		writeto (bool): If True, will write 2D model PSF (differentiated with '_topsf' 
 			suffix) and will amend relevant image WCS information/remove extraneous extensions. 
 			This is in addition to the .psf file saved by PSFEx. No 2D PSF model is saved by PSFEx 
 			by default, but this can be toggled in the tools.pypsfex arguments.
+		clobber (bool): If True, will overwrite existing files with the same name on save.
+			(Default state -- clobber = False -- is recommended.)
 		savepsfex (str): If 'fits' or 'arr' save 2D model to that format.
 		seconf (str): Path to SExtractor configuration file if not using default.
 		peconf (str): Path to PSFEx configuration file if not using default.
@@ -824,6 +891,17 @@ def psfex(coords, img, imcam, pos, plot = False, verbose = False, writeto = True
 	Returns:
 		2D PSFEx PSF model
 	"""
+	if type(coords) == str:
+		coords = tools.objloc(coords)
+
+	if not pos:
+		im_cam = imcam.split('/')
+		inst = im_cam[0]
+		if len(im_cam) == 1:
+			camera = None
+		if len(im_cam) == 2:
+			camera = im_cam[1]
+		pos = tools.checkpixloc(coords, img, inst, camera)
 
 	ext = 1 #run SExtractor only on relevant SCI extension
 	extv = 1
@@ -838,7 +916,7 @@ def psfex(coords, img, imcam, pos, plot = False, verbose = False, writeto = True
 		extv = pos[2]
 
 	if mask:
-		tools.mask_fits(img, extv, **maskparams)
+		tools.mask_fits(img, extv, clobber = clobber, **maskparams)
 		if verbose:
 			print('Finished masking data, running SExtractor')
 		tools.pysextractor(img.replace('.fits', '_mask.fits')+'[%i]'%ext, config = seconf)	
@@ -853,9 +931,9 @@ def psfex(coords, img, imcam, pos, plot = False, verbose = False, writeto = True
 		print('Finished PSFEx, generating image')
 
 	coordstring = str(coords.ra)
-	if coords.dec.deg > 0:
-		coordstring += '+'+str(coords.dec)
 	if coords.dec.deg >= 0:
+		coordstring += '+'+str(coords.dec)
+	if coords.dec.deg < 0:
 		coordstring += str(coords.dec)
 
 	modname = img.replace('.fits', '_'+coordstring+'_%s'%pos[3]+'_psf')
@@ -873,7 +951,7 @@ def psfex(coords, img, imcam, pos, plot = False, verbose = False, writeto = True
 	if writeto:
 		if verbose:
 			print('Writing to %s.fits.'%modname.replace('_psf', '_topsf'))
-		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'PSFEx')
+		tools.rewrite_fits(psfmodel, coords, img, imcam, pos, method = 'PSFEx', clobber = clobber)
 
 	return psfmodel
 
