@@ -1,4 +1,5 @@
 from asdf import AsdfFile
+import astropy
 from astropy.coordinates import SkyCoord, name_resolve
 import astropy
 import astropy.units as u
@@ -31,22 +32,27 @@ def objloc(obj):
 	Returns:
 		coords (astropy coordinates object)
 	"""
-	isname = False #check if obj is name or coordinates
-	for s in obj:
-		if s.isalpha():
-			isname = True
-			break
 
-	if isname:
-		coords = name_resolve.get_icrs_coordinates(obj)
+	if type(obj) == astropy.coordinates.sky_coordinate.SkyCoord:
+		return obj
 
-	if not isname:
-		if ':' in obj:
-			coords = SkyCoord(obj, unit = (u.hour, u.deg), frame = 'icrs')
-		if not ':' in obj:
-			coords = SkyCoord(obj, unit = u.deg, frame = 'icrs')
+	else:
+		isname = False #check if obj is name or coordinates
+		for s in obj:
+			if s.isalpha():
+				isname = True
+				break
 
-	return coords
+		if isname:
+			coords = name_resolve.get_icrs_coordinates(obj)
+
+		if not isname:
+			if ':' in obj:
+				coords = SkyCoord(obj, unit = (u.hour, u.deg), frame = 'icrs')
+			if not ':' in obj:
+				coords = SkyCoord(obj, unit = u.deg, frame = 'icrs')
+
+		return coords
 
 
 def checkpixloc(coords, img, inst, camera = None):
@@ -126,6 +132,7 @@ def checkpixloc(coords, img, inst, camera = None):
 			out = [float(x_coord), float(y_coord), chip, filt]
 
 	if imcam in ['WFPC', 'WFPC1', 'WFPC2']:
+		# make chip indexing explicit for consistency with other instruments
 		if imcam in ['WFPC', 'WFPC1']: #accounting for use of both names
 			chip1 = hdu[1]
 			chip2 = hdu[2]
@@ -230,13 +237,15 @@ def checkpixloc(coords, img, inst, camera = None):
 	#add support for FOC and NIRISS AMI
 
 
-def to_asdf(fitspath, save = True):
+def to_asdf(fitspath, save = True, clobber = False):
 	"""
 	Convert .fits file to .asdf by simply wrapping data and header extensions.
 
 	Parameters:
 		fitspath (str): Path to .fits file to convert.
 		save (bool): If True, saves the .asdf file.
+		clobber (bool): If True, will overwrite existing files with the same name on save.
+			(Default state -- clobber = False -- is recommended.)
 
 	Returns:
 		ASDF file object
@@ -254,7 +263,13 @@ def to_asdf(fitspath, save = True):
 		asdf_out['dat'+str(i)] = fits_in[i].data
 
 	if save:
-		asdf_out.write_to(fitspath.replace('fits', 'asdf'))
+		if clobber:
+			asdf_out.write_to(fitspath.replace('fits', 'asdf'))
+		if not clobber:
+			if os.path.exists(fitspath.replace('fits', 'asdf')):
+				warnings.warn('File already exists; proceeding without saving. Use clobber = True to allow overwrite.', Warning, stacklevel = 2)
+			if not os.path.exists(fitspath.replace('fits', 'asdf')):
+				asdf_out.write_to(fitspath.replace('fits', 'asdf'))
 
 	return asdf_out
 
@@ -312,7 +327,7 @@ def regridarr (im, sample):
 
 	Parameters:
 		im (arr): PSF image array.
-		sample (float): If sample > 1, oversampled; if sample < 1, undersampled.
+		sample (float): Npix,im/Npix,orig. If sample > 1, oversampled; if sample < 1, undersampled.
 
 	Returns:
 		Interpolated and regridded PSF model.
@@ -323,8 +338,8 @@ def regridarr (im, sample):
 		return im
 
 	x,y = im.shape
-	xnew = np.arange(0, x, 1/sample)
-	ynew = np.arange(0, y, 1/sample)
+	xnew = np.arange(0, x, sample)
+	ynew = np.arange(0, y, sample)
 
 	spline = RectBivariateSpline(np.arange(x), np.arange(y), im)
 	out = spline(xnew[xnew <= x-1], ynew[ynew <= y-1])
@@ -332,7 +347,7 @@ def regridarr (im, sample):
 	return out	
 
 
-def psfexim(psfpath, pixloc, regrid = True, save = False):
+def psfexim(psfpath, pixloc, regrid = True, save = False, clobber = False):
 	"""
 	Generate image from PSFEx .psf file.
 
@@ -342,6 +357,8 @@ def psfexim(psfpath, pixloc, regrid = True, save = False):
 		regrid (bool): If True, will (interpolate and) regrid model PSF to image pixel scale.
 		save (str): If 'fits' or 'arr', will save in the specified format with name from psfpath.
 			The option to save as an array results in a .npy file.
+		clobber (bool): If True (and save = True), will overwrite existing files with the same name.
+			(Default state -- clobber = False -- is recommended.)
 
 	Returns:
 		2D image of PSFEx model
@@ -383,15 +400,21 @@ def psfexim(psfpath, pixloc, regrid = True, save = False):
 		if save.lower() not in ['arr', 'fits']:
 			warnings.warn('Save input must be "arr" or "fits". Generating PSF model without saving.', Warning, stacklevel = 2)
 		if save.lower() == 'arr':
-			np.save(psfpath.replace('.psf', '_psfex_psf.npy'), psfmodel)
+			if clobber:
+				np.save(psfpath.replace('.psf', '_psfex_psf.npy'), psfmodel)
+			if not clobber:
+				if os.path.exists(psfpath.replace('.psf', '_psfex_psf.npy')):
+					warnings.warn('File already exists; generating PSF model without saving. Use clobber = True to allow overwrite.', Warning, stacklevel = 2)
+				if not os.path.exists(psfpath.replace('.psf', '_psfex_psf.npy')):
+					np.save(psfpath.replace('.psf', '_psfex_psf.npy'), psfmodel)
 		if save.lower() == 'fits':
-			fits.writeto(psfpath.replace('.psf', '_psfex_psf.fits'), psfmodel)
+			fits.writeto(psfpath.replace('.psf', '_psfex_psf.fits'), psfmodel, overwrite = clobber)
 
 	return psfmodel
 
 
 def pypsfex(cat_path, pos, config = None, userargs = None, makepsf = True, 
-	savepsf = False, keepconfig = False, regrid = True):
+	savepsf = False, keepconfig = False, regrid = True, clobber = False):
 	"""
 	Wrapper to easily call PSFEx from python. 
 
@@ -407,6 +430,8 @@ def pypsfex(cat_path, pos, config = None, userargs = None, makepsf = True,
 		savepsf (str): If 'fits', 'arr', or 'txt', will save 2D model PSF in that file format with the same name as the catalog.
 		keepconfig (str): If True, retain parameter files and convolutional kernels moved to working dir.
 		regrid (bool): If True, will (interpolate and) regrid model PSF to image pixel scale.
+		clobber (bool): If True, will overwrite existing files with the same name on save.
+			(Default state -- clobber = False -- is recommended.)
 
 	Returns:
 		Generates a .psf file that stores linear bases of PSF.
@@ -443,15 +468,16 @@ def pypsfex(cat_path, pos, config = None, userargs = None, makepsf = True,
 			x, y = im.shape/2 #select central pixel if not specified
 
 
-		psfmodel = psfexim(cat_path.replace('cat', 'psf'), pixloc = (x, y), save = savepsf, regrid = regrid)
+		psfmodel = psfexim(cat_path.replace('cat', 'psf'), pixloc = (x, y), 
+			save = savepsf, regrid = regrid, clobber = clobber)
 			
 		return psfmodel
 
 
-def rewrite_fits(psfarr, coords, img, imcam, pos, method = None):
+def rewrite_fits(psfarr, coords, img, imcam, pos, method = None, clobber = False):
 	"""
 	Write relevant image headers to the model PSFs and modify the coordinates and WCS.
-	Creates a full _topsf.fits file with only one SCI extension for use with drizzle/resample.
+	Creates a full _topsf_*.fits file with only one SCI extension for use with drizzle/resample.
 
 	Parameters:
 		psfarr (arr): The 2D PSF model.
@@ -460,6 +486,8 @@ def rewrite_fits(psfarr, coords, img, imcam, pos, method = None):
 		imcam (str): 'ACS/WFC' is the only recommended instrument/camera combination for this PSF generation method.
 		pos (list): Location of object of interest (spatial and spectral).[X, Y, chip, filter]
 		method (list): Method used to generate PSF.
+		clobber (bool): If True, will overwrite existing FITS files with the same name.
+			(Default state -- clobber = False -- is recommended.)
 
 	Returns: 
 		Generates a new FITS file with a _topsf suffix, which stores the 2D PSF model in the 
@@ -482,15 +510,49 @@ def rewrite_fits(psfarr, coords, img, imcam, pos, method = None):
 	imgdat = fits.open(img)
 
 	psfim = np.zeros_like(imgdat[ext].data)
-	xmin = int(pos[0]) - psfarr.shape[0]//2
-	xmax = int(pos[0]) + psfarr.shape[0]//2
-	ymin = int(pos[1]) - psfarr.shape[1]//2
-	ymax = int(pos[1]) + psfarr.shape[1]//2
+	xmin = int(pos[0]) - psfarr.shape[1]//2
+	xmax = int(pos[0]) + psfarr.shape[1]//2
+	ymin = int(pos[1]) - psfarr.shape[0]//2
+	ymax = int(pos[1]) + psfarr.shape[0]//2
+
+	## to deal with PSFs near edge of frame
+	update_xmax = 1 #whether to use to adjust shape
+	update_ymax = 1
+
+	if xmin < 0:
+		psfarr = psfarr[:, int(abs(xmin))+1:]
+		xmin = 0
+
+	if ymin < 0:
+		psfarr = psfarr[int(abs(ymin))+1:, :]
+		ymin = 0
+
+	if xmax > psfim.shape[1]:
+		over = xmax - psfim.shape[1]
+		psfarr = psfarr[:, :int(over)]
+		xmax = psfim.shape[1]
+		update_xmax = 0
+
+	if ymax > psfim.shape[0]:
+		over = ymax - psfim.shape[0]
+		psfarr = psfarr[:int(over), :]
+		ymax = psfim.shape[0]
+		update_ymax = 0
+
+	if psfarr.shape[1] % 2 != psfim[ymin:ymax, xmin:xmax].shape[1] % 2:
+		if update_xmax == 1:
+			xmax += 1
+		if update_xmax == 0:
+			xmin -= 1
+
+	if psfarr.shape[0] % 2 != psfim[ymin:ymax, xmin:xmax].shape[0] % 2:
+		if update_ymax == 1:
+			ymax += 1
+		if update_ymax == 0:
+			ymin -= 1
+
+	psfim[ymin:ymax, xmin:xmax] += psfarr
 
-	if psfarr.shape[0] % 2 == 0:
-		psfim[ymin:ymax, xmin:xmax] += psfarr
-	if psfarr.shape[0] % 2 != 0:
-		psfim[ymin:ymax+1, xmin:xmax+1] += psfarr
 
 	cphdr = fits.PrimaryHDU(header = imgdat[0].header)
 
@@ -499,21 +561,22 @@ def rewrite_fits(psfarr, coords, img, imcam, pos, method = None):
 		hdr['COMMENT'] = "PSF generated using %s via spike."%method
 	if not method:
 		hdr['COMMENT'] = "PSF generated via spike."
-	cihdr = fits.ImageHDU(data = psfim, header = hdr, name = 'SCI')
+	cihdr = fits.ImageHDU(data = psfim, header = hdr, name = 'SCI', ver = 1)
 
 	if img.split('_')[-1] != '_c0m.fits':
 		ehdrdat = np.zeros_like(imgdat[('ERR', extv)].data) #shouldn't matter, but doing this explicitly anyway
 		dqhdrdat = np.zeros_like(imgdat[('DQ', extv)].data)
-		cehdr = fits.ImageHDU(data = ehdrdat, header = imgdat[('ERR', extv)].header, name = 'ERR')
-		cdqhdr = fits.ImageHDU(data = dqhdrdat, header = imgdat[('DQ', extv)].header, name = 'DQ')
+		cehdr = fits.ImageHDU(data = ehdrdat, header = imgdat[('ERR', extv)].header, name = 'ERR', ver = 1)
+		cdqhdr = fits.ImageHDU(data = dqhdrdat, header = imgdat[('DQ', extv)].header, name = 'DQ', ver = 1)
 
 	coordstring = str(coords.ra)
-	if coords.dec.deg > 0:
-		coordstring += '+'+str(coords.dec)
 	if coords.dec.deg >= 0:
+		coordstring += '+'+str(coords.dec)
+	if coords.dec.deg < 0:
 		coordstring += str(coords.dec)
 
-	modname = img.replace('.fits', '_'+coordstring+'_%s'%pos[3]+'_topsf.fits')
+	img_type = img.split('_')[-1].replace('.fits', '')
+	modname = img.replace('%s.fits'%img_type, coordstring+'_%s'%pos[3]+'_topsf_%s.fits'%img_type)
 
 	if img.split('_')[-1] != '_c0m.fits':
 		hdlist = [cphdr, cihdr, cehdr, cdqhdr]
@@ -542,23 +605,23 @@ def rewrite_fits(psfarr, coords, img, imcam, pos, method = None):
 		d2im2 = None
 
 
-	if (d2im1 == 'EXTVER: 1') & (d2im2 == 'EXTVER: 2'):
+	if (d2im1 in ['EXTVER: 1', 'EXTVER: 1.0']) & (d2im2 in ['EXTVER: 2', 'EXTVER: 2.0']):
 		hdlist.append(fits.ImageHDU(data = imgdat[('D2IMARR', 1)].data, header = imgdat[('D2IMARR', 1)].header, 
 			name = 'D2IMARR', ver = 1))
 		hdlist.append(fits.ImageHDU(data = imgdat[('D2IMARR', 2)].data, header = imgdat[('D2IMARR', 2)].header, 
 			name = 'D2IMARR', ver = 2))
-	if (d2im1 == 'EXTVER: 3') & (d2im2 == 'EXTVER: 4'):
+	if (d2im1 in ['EXTVER: 3', 'EXTVER: 3.0']) & (d2im2 in ['EXTVER: 4', 'EXTVER: 4.0']):
 		hdlist.append(fits.ImageHDU(data = imgdat[('D2IMARR', 3)].data, header = imgdat[('D2IMARR', 3)].header, 
 			name = 'D2IMARR', ver = 3))
 		hdlist.append(fits.ImageHDU(data = imgdat[('D2IMARR', 4)].data, header = imgdat[('D2IMARR', 4)].header, 
 			name = 'D2IMARR', ver = 4))
 
-	if (dp1 == 'EXTVER: 1') & (dp2 == 'EXTVER: 2'):
+	if (dp1 in ['EXTVER: 1', 'EXTVER: 1.0']) & (dp2 in ['EXTVER: 2', 'EXTVER: 2.0']):
 		hdlist.append(fits.ImageHDU(data = imgdat[('WCSDVARR', 1)].data, header = imgdat[('WCSDVARR', 1)].header, 
 			name = 'WCSDVARR', ver = 1))
 		hdlist.append(fits.ImageHDU(data = imgdat[('WCSDVARR', 2)].data, header = imgdat[('WCSDVARR', 2)].header, 
 			name = 'WCSDVARR', ver = 2))
-	if (dp1 == 'EXTVER: 3') & (dp2 == 'EXTVER: 4'):
+	if (dp1 in ['EXTVER: 3', 'EXTVER: 3.0']) & (dp2 in ['EXTVER: 4', 'EXTVER: 4.0']):
 		hdlist.append(fits.ImageHDU(data = imgdat[('WCSDVARR', 3)].data, header = imgdat[('WCSDVARR', 3)].header, 
 			name = 'WCSDVARR', ver = 3))
 		hdlist.append(fits.ImageHDU(data = imgdat[('WCSDVARR', 4)].data, header = imgdat[('WCSDVARR', 4)].header, 
@@ -574,16 +637,16 @@ def rewrite_fits(psfarr, coords, img, imcam, pos, method = None):
 	hdulist = fits.HDUList(hdlist)
 
 	if img.split('_')[-1] != '_c0m.fits':
-		hdulist.writeto(modname)
+		hdulist.writeto(modname, overwrite = clobber)
 
 	if img.split('_')[-1] == '_c0m.fits':
 		modname = modname.replace('_topsf.fits', '_topsf_c0m.fits')
-		hdulist.writeto(modname)
+		hdulist.writeto(modname, overwrite = clobber)
 		os.system('cp %s %s'%(img.replace('c0m.fits', 'c1m.fits'), modname.replace('c0m.fits', 'c1m.fits')))
 
 
-def mask_fits(img, ext = 1, maskdq = True, dqthresh = 0, 
-	maskerr = False, errthresh = 20, usermask = None, fillval = 0):
+def mask_fits(img, ext = 1, maskdq = True, dqthresh = 0, maskerr = False, 
+	errthresh = 20, usermask = None, fillval = 0, clobber = False):
 	"""
 	Generate a FITS file that fills in masked pixels with a specified value. Useful for
 	feeding to e.g., SExtractor. Preserves truncated FITS extension structure, for the specified
@@ -601,6 +664,8 @@ def mask_fits(img, ext = 1, maskdq = True, dqthresh = 0,
 		usermask (arr): If specified, used as mask on data array (good pixels should have value of 0). 
 			Must be the same dimensions as the data array. (Can be used in addition to DQ and ERR masking.)
 		fillval (float): Value with which to fill the masked pixels.
+		clobber (bool): If True , will overwrite existing FITS files with the same name.
+			(Default state -- clobber = False -- is recommended.)
 
 	Returns: 
 		Generates a new FITS file with a _mask suffix with masked pixels filled in by fillval.
@@ -653,10 +718,10 @@ def mask_fits(img, ext = 1, maskdq = True, dqthresh = 0,
 	hdlist = [cphdr, cihdr, cehdr, cdqhdr]
 
 	hdulist = fits.HDUList(hdlist)
-	hdulist.writeto(img.replace('.fits', '_mask.fits'))
+	hdulist.writeto(img.replace('.fits', '_mask.fits'), overwrite = clobber)
 
 
-def cutout(img, coords, ext = 1, fov_pixel = 120, save = True):
+def cutout(img, coords, ext = 1, fov_pixel = 120, save = True, clobber = False):
 	"""
 	Get cutout of image around some coordinates.
 
@@ -665,8 +730,12 @@ def cutout(img, coords, ext = 1, fov_pixel = 120, save = True):
 		coords (astropy skycoords object): Coordinates of object of interest or list of skycoord objects.
 			Easiest is to feed in the output of spike.tools.objloc.
 		ext (int): Integer index of extension to crop.
-		fov_pixel (int): Diameter of square cutout region in pixels.
-		save (bool): If True, will save a FITS file containing cropped region.
+		fov_pixel (int): "Diameter" of square cutout region in pixels.
+		save (bool): If True, will save a FITS file containing cropped region. Note that the 
+			output FITS file will not have any distortion corrections stored as it is already 
+			decontextualized from the original image.
+		clobber (bool): If True (and save = True), will overwrite existing FITS files with the same name.
+			(Default state -- clobber = False -- is recommended.)
 
 	Returns: 
 		cutoutim (arr): Array containing cutout region of the image.
@@ -693,8 +762,26 @@ def cutout(img, coords, ext = 1, fov_pixel = 120, save = True):
 	ymin = y0 - fov_pixel//2
 	ymax = y0 + fov_pixel//2
 
+	xcen = fov_pixel//2
+	ycen = fov_pixel//2
+
+	## to deal with PSFs near edge of frame
+	if xmin < 0:
+		xcen += xmin
+		xmin = 0
+
+	if ymin < 0:
+		ycen += ymin
+		ymin = 0
+
+	if xmax > dat.shape[1]:
+		xmax = dat.shape[1]
+
+	if ymax > dat.shape[0]:
+		ymax = dat.shape[0]
+
 	if fov_pixel % 2 == 0:
-		coords0 = utils.pixel_to_skycoords((x0, y0), wcs)
+		coords0 = utils.pixel_to_skycoord(x0, y0, wcs)
 		ra = coords0.ra.deg
 		dec = coords0.dec.deg
 		cutoutim = dat[ymin:ymax, xmin:xmax]
@@ -705,17 +792,37 @@ def cutout(img, coords, ext = 1, fov_pixel = 120, save = True):
 
 	hdr['CRVAL1'] = ra
 	hdr['CRVAL2'] = dec
-	hdr['CRPIX1'] = fov_pixel//2
-	hdr['CRPIX2'] = fov_pixel//2
-	hdr['NAXIS1'] = fov_pixel
-	hdr['NAXIS2'] = fov_pixel
+	hdr['CRPIX1'] = xcen//2
+	hdr['CRPIX2'] = ycen//2
+	hdr['NAXIS1'] = cutoutim.shape[1]
+	hdr['NAXIS2'] = cutoutim.shape[0]
+
+	# remove WCSDVARR and D2IMARR keys, since output isn't really on original
+	# image grid any longer
+	try:
+		del hdr['DP1']
+		del hdr['DP2']
+		del hdr['D2IM1']
+		del hdr['D2IM2']
+		del hdr['D2IMDIS1']
+		del hdr['D2IMDIS2']
+		del hdr['D2IMERR1']
+		del hdr['D2IMERR2']
+		del hdr['CPDIS1']
+		del hdr['CPDIS2']
+		del hdr['CPERR1']
+		del hdr['CPERR2']
+	except:
+		pass
 
 	cihdr = fits.ImageHDU(data = cutoutim, header = hdr, name = 'SCI')
 
 	hdlist = [cphdr, cihdr]
+
+
 	hdulist = fits.HDUList(hdlist)
 	if save:
-		hdulist.writeto(img.replace('.fits', '_crop.fits'))
+		hdulist.writeto(img.replace('.fits', '_crop.fits'), overwrite = clobber)
 
 	return cutoutim
 
